/*
 * (!c) 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *   Support more hardware/protocol types
 *   Support newer ARP operation codes
 *   Support IPv6
 */

#include <stdio.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include "net/ether_types.h"
#include "net/arp.h"

#include "types.h"

void dump_hex(const u8 *packet, int length);
char *my_ether_ntoa(const u8 *addr);
char *ip2name(u32 ip);

#ifndef ARP_HDRLEN
#define ARP_HDRLEN	8
#endif

#ifndef ETHER_ADDR_LEN
#define ETHER_ADDR_LEN	6
#endif

#ifndef IP_ADDR_LEN
#define IP_ADDR_LEN	4
#endif

void dump_arp(const u8 *packet, int length)
{
    struct arp_hdr *arp = (struct arp_hdr *) packet;
    struct ether_arp *ea = (struct ether_arp *) arp;
    u16 ar_hrd;		/* hardware type */
    u16 ar_pro;		/* protocol type */
    int i;
    extern int numeric;
    extern int resolve;
    
    printf("  ARP: ");
    
#ifdef DEBUG
    if (! packet) {
	printf("DEBUG: We got no packet!\n");
	return;
    }
#endif

    /* Check the length received */
    if (length < ARP_HDRLEN) {
	printf("ERROR: Truncated ARP: %u\n", length);
	goto bad;
    }

    /* Check the hardware and protocol address length */
    if (length < ARP_HDRLEN + (arp->ar_hln << 1) + (arp->ar_pln << 1)) {
	printf("ERROR: Truncated ARP: %u hln=%u pln=%u\n",
	       length, arp->ar_hln, arp->ar_pln);
	goto bad;
    }    
    
    ar_hrd = ntohs(arp->ar_hrd);
    ar_pro = ntohs(arp->ar_pro);

    /* Print the hardware type */
    printf("hrd=");
    if (numeric) {
	printf("%04x ", ar_hrd);
    }
    else {
	switch (ar_hrd) {
	case ARPHRD_ETHER:
	    printf("ethernet ");
	    break;
	default:
	    printf("%04x ", ar_hrd);
	}
    }
    
    /* Print the protocol type */
    printf("pro=");
    if (numeric) {
	printf("%04x ", ar_pro);
    }
    else {
	switch (ar_pro) {
	case ETHERTYPE_IP:
	    printf("ip ");
	    break;
	case ETHERTYPE_IPV6:
	    printf("ip6 ");
	    break;
	default:
	    printf("%04x ", ar_pro);
	}
    }

    /* Print the operation code */
    printf("op=");
    if (numeric) {
	printf("%04x ", ntohs(arp->ar_op));
    }
    else {
	switch (ntohs(arp->ar_op)) {
	case ARPOP_REQUEST:
	    printf("arp-request ");
	    break;
	case ARPOP_REPLY:
	    printf("arp-reply ");
	    break;
	case ARPOP_RREQUEST:
	    printf("rarp-request ");
	    break;
	case ARPOP_RREPLY:
	    printf("rarp-reply ");
	    break;
	case ARPOP_IREQUEST:
	    printf("iarp-request ");
	    break;
	case ARPOP_IREPLY:
	    printf("iarp-reply ");
	    break;
	default:
	    printf("%04x ", ntohs(arp->ar_op));	    
	}
    }

    packet += ARP_HDRLEN;
    length -= ARP_HDRLEN;

    if ((ar_hrd == ARPHRD_ETHER) && (ar_pro == ETHERTYPE_IP)) {
	if ((arp->ar_hln != ETHER_ADDR_LEN) || (arp->ar_pln != IP_ADDR_LEN)) {
	    printf("ERROR: Invalid address length: hln=%u pln=%u\n",
		   arp->ar_hln, arp->ar_pln);
	    goto bad;
	}
	/* Print sender ethernet address */
	printf("sha=%s ", my_ether_ntoa(ea->arp_sha));
	/* Print sender ip address */
	printf("spa=");
	if (resolve) {
	    printf("%s ", ip2name(*(u32 *)&ea->arp_spa));
	}
	else {
	    printf("%s ", inet_ntoa(*(struct in_addr *)&ea->arp_spa));
	}
	/* Print target ethernet address */
	printf("tha=%s ", my_ether_ntoa(ea->arp_tha));
	/* Print target ip address */
	printf("tpa=");
	if (resolve) {
	    printf("%s ", ip2name(*(u32 *)&ea->arp_tpa));
	}
	else {
	    printf("%s ", inet_ntoa(*(struct in_addr *)&ea->arp_tpa));
	}	
    }
    else {
	/* Print sender hardware address */
	printf("sha=");
	for (i = 0; i < arp->ar_hln; i++) {
	    printf("%02x", *(packet + i));
	}
	putchar (' ');
	packet += i;
	/* Print sender protocol address */
	printf("spa=");
	for (i = 0; i < arp->ar_pln; i++) {
	    printf("%02x", *(packet + i));
	}
	putchar(' ');
	packet += i;
	/* Print target hardware address */
	printf("tha=");
	for (i = 0; i < arp->ar_hln; i++) {
	    printf("%02x", *(packet + i));
	}
	putchar(' ');
	packet += i;
	/* Print target protocol address */
	printf("tpa=");
	for (i = 0; i < arp->ar_pln; i++) {
	    printf("%02x ", *(packet + i));
	}
/*	packet += i; */
    }

    putchar('\n');
    
    /* Ignore Ethernet padding */
    
    return;
    
bad:
    /* Hex dump bad packets */
    dump_hex(packet, length);
    
    return;    
}
