/*
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#include <stdio.h>

#include <netinet/in.h>		/* ntohs() */

#include "net/ether_types.h"
#include "net/ether.h"

#include "types.h"

void dump_hex(const u8 *packet, int length);
void dump_ip (const u8 *packet, int length);
#if 0
void dump_ip6(const u8 *packet, int length);
#endif
void dump_arp(const u8 *packet, int length);
char *my_ether_ntoa(const u8 *addr);

#ifndef ETHER_HDRLEN
#define ETHER_HDRLEN	14
#endif

void dump_ether(const u8 *packet, int length)
{
    struct ether_hdr *ether = (struct ether_hdr *) packet;
#ifdef __linux__    
    extern int linux_loopback;
#endif        

    /* Don't print any info if we are on the loopback interface */
#ifdef __linux__
    if (! linux_loopback)
#endif
    {
	printf("ETHER: ");
    }
    
#ifdef DEBUG
    if (! packet) {
	printf("DEBUG: We got no packet!\n");
	return;
    }
#endif

    /* Check the length received */
    if (length < ETHER_HDRLEN) {
	printf("ERROR: Truncated Ethernet: %u\n", length);
	dump_hex(packet, length);
	return;
    }
    
    /* Don't print any info if we are on the loopback interface */    
#ifdef __linux__
    if (! linux_loopback)
#endif
    {    
	printf("%s", my_ether_ntoa(ether->ether_shost));
	printf(" -> %s ", my_ether_ntoa(ether->ether_dhost));
    }

    packet += ETHER_HDRLEN;
    length -= ETHER_HDRLEN;
    
    switch (ntohs(ether->ether_type)) {
    case ETHERTYPE_IP:
	/* Don't print any info if we are on the loopback interface */	
#ifdef __linux__
	if (! linux_loopback)
#endif
	{    	
	    printf("ip\n");
	}
	dump_ip(packet, length);
	break;
#if 0
    case ETHERTYPE_IPV6:
	/* Don't print any info if we are on the loopback interface */	
#ifdef __linux__
	if (! linux_loopback)
#endif
	{    	
	    printf("ip6\n");
	}	
	dump_ip6(packet, length);
	break;
#endif	
    case ETHERTYPE_ARP:
	printf("arp\n");
	dump_arp(packet, length);
	break;
    case ETHERTYPE_REVARP:
	printf("rarp\n");
	dump_arp(packet, length);
	break;
    default:
	printf("type=%04x\n", ntohs(ether->ether_type));
	dump_hex(packet, length);
    }
    
    return;
}
