/*
 * (!c) 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *   Support newer ICMP types
 *   XXX's
 */

#include <stdio.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include "net/icmp.h"

#include "types.h"

void dump_hex(const u8 *packet, int length);
u16 in_cksum(u16 *addr, int length);
char *ts2asc(u32 ts);

static char *icmp_types[] = {
    "echo-reply",
    "1", "2",
    "destination-unreachable",
    "source-quench",
    "redirect",
    "6", "7",    
    "echo-request",
    "router-advertisement",
    "router-solicitacion",
    "time-exceeded",
    "parameter-problem",
    "timestamp-request",
    "timestamp-reply",
    "information-request",
    "information-reply",
    "address-mask-request",
    "address-mask-reply",
    "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29",
    "traceroute"
};

static char *unreach_codes[] = {
    "unreachable-network",
    "unreachable-host",
    "unreachable-protocol",
    "unreachable-port",
    "fragmentation-needed-but-df-set",
    "source-route-failed",
    "unknown-network",
    "unknown-host",
    "source-host-isolated",
    "prohibited-network",
    "prohibited-host",
    "bad-tos-for-network",
    "bad-tos-for-host",
    "filtered",
    "host-precedence-violation",
    "precedence-cutoff"
};

static char *redirect_codes[] = {
    "network",
    "host",
    "tos-and-network",
    "tos-and-host"
};

static char *timxceed_codes[] = {
    "ttl-equals-zero-during-transit",
    "ttl-equals-zero-during-reassembly"
};

static char *paramprob_codes[] = {
    "bad-ip-header",
    "required-option-missing",
    "bad-length"
};

void dump_icmp(const u8 *packet, int length)
{
    struct icmp_hdr *icmp = (struct icmp_hdr *) packet;
    u16 cksum;
    int i;
    extern int numeric;
    
    printf(" ICMP: ");
    
#ifdef DEBUG    
    if (! packet) {
	printf("DEBUG: We got no packet!\n");
	return;
    }
#endif	    
    
    /* Check the length received */
    if (length < 8) {
	printf("ERROR: Truncated ICMP: %u\n", length);
	goto bad;
    }
    
    /* Checksum */
    cksum = in_cksum((u16 *) icmp, length);
    if (cksum != 0) {
	printf("ERROR: Bad ICMP checksum\n");
	goto bad;
    }

    printf("type=");
    if (numeric) {
	printf("%u ", icmp->icmp_type);
    }
    else {
	if (icmp->icmp_type <= ICMP_MAXTYPE) {
	    printf("%s ", icmp_types[icmp->icmp_type]);
	}
	else {
	    printf("%u ", icmp->icmp_type);	    
	}
    }
    
    printf("code=");
    if (numeric) {
	printf("%u ", icmp->icmp_code);
    }
    else {
	switch (icmp->icmp_type) {
	case ICMP_UNREACH:
	    if (icmp->icmp_code < 16) {
		printf("%s ", unreach_codes[icmp->icmp_code]);
	    }
	    else {
		printf("%u ", icmp->icmp_code);
	    }
	    break;
	case ICMP_REDIRECT:
	    if (icmp->icmp_code < 4) {
		printf("%s ", redirect_codes[icmp->icmp_code]);
	    }
	    else {
		printf("%u ", icmp->icmp_code);
	    }
	    break;
	case ICMP_TIMXCEED:
	    if (icmp->icmp_code < 2) {
		printf("%s ", timxceed_codes[icmp->icmp_code]);
	    }
	    else {
		printf("%u ", icmp->icmp_code);
	    }
	    break;
	case ICMP_PARAMPROB:
	    if (icmp->icmp_code < 3) {
		printf("%s ", paramprob_codes[icmp->icmp_code]);
	    }
	    else {
		printf("%u ", icmp->icmp_code);
	    }
	    break;
	default:
	    printf("%u ", icmp->icmp_code);
	}
    }
    
    switch (icmp->icmp_type) {
    case ICMP_UNREACH:
	if (icmp->icmp_code == ICMP_UNREACH_NEEDFRAG) {
	    if (icmp->icmp_pmvoid) {
		printf("void=%04x ", ntohs(icmp->icmp_pmvoid));
	    }
	    printf("nextmtu=%04x", ntohs(icmp->icmp_nextmtu));
	}
	else {
	    if (icmp->icmp_void) {
		printf("void=%08x", ntohl(icmp->icmp_void));
	    }
	}
	packet += 8;
	length -= 8;
	break;	    
    case ICMP_TIMXCEED:
    case ICMP_SOURCEQUENCH:
    case ICMP_ROUTERSOLICIT:
	if (icmp->icmp_void) {
	    printf("void=%08x", ntohl(icmp->icmp_void));
	}
	packet += 8;
	length -= 8;
	break;
    case ICMP_PARAMPROB:
	printf("ptr=%02x", icmp->icmp_pptr);
	if (ntohl(icmp->icmp_void) & 0x00ffffff) {
	    printf(" void=%06x", ntohl(icmp->icmp_void) & 0x00ffffff);
	}
	packet += 8;
	length -= 8;
	break;
    case ICMP_REDIRECT:
	printf("gwaddr=%s", inet_ntoa(*(struct in_addr *)&icmp->icmp_gwaddr));
	packet += 8;
	length -= 8;
	break;
    case ICMP_ECHO:
    case ICMP_ECHOREPLY:
    case ICMP_IREQ:
    case ICMP_IREQREPLY:
	/* XXX - Mike Muuss' ping(1) does not htons() icmp_id & icmp_seq */
	printf("id=%04x seq=%04x",
	       ntohs(icmp->icmp_id), ntohs(icmp->icmp_seq));
	packet += 8;
	length -= 8;
	break;
    case ICMP_TSTAMP:
    case ICMP_TSTAMPREPLY:
	printf("id=%04x seq=%04x",
	       ntohs(icmp->icmp_id), ntohs(icmp->icmp_seq));
	if (length < 20) {
	    printf("ERROR: Truncated ICMP: %u\n", length);
	    goto bad;
	}
	printf(" otime=");
	if (ntohl(icmp->icmp_otime) & 0x80000000) {
	    printf("%08x", ntohl(icmp->icmp_otime));
	}
	else {
	    printf("%s", ts2asc(icmp->icmp_otime));
	}
	printf(" rtime=");
	if (ntohl(icmp->icmp_rtime) & 0x80000000) {
	    printf("%08x", ntohl(icmp->icmp_rtime));
	}
	else {
	    printf("%s", ts2asc(icmp->icmp_rtime));
	}
	printf(" ttime=");
	if (ntohl(icmp->icmp_ttime) & 0x80000000) {
	    printf("%08x", ntohl(icmp->icmp_ttime));
	}
	else {
	    printf("%s", ts2asc(icmp->icmp_ttime));
	}	
	packet += 20;
	length -= 20;
	break;
    case ICMP_MASKREQ:
    case ICMP_MASKREPLY:
	printf("id=%04x seq=%04x ",
	       ntohs(icmp->icmp_id), ntohs(icmp->icmp_seq));
	if (length < 12) {
	    printf("ERROR: Truncated ICMP: %u\n", length);
	    goto bad;
	}
	printf("mask=%s", inet_ntoa(*(struct in_addr *)&icmp->icmp_mask));
	packet += 12;
	length -= 12;
	break;
    case ICMP_ROUTERADVERT:
/*	printf("naddr=%u ", icmp->icmp_num_addrs); */
/*	printf("esize=%u ", icmp->icmp_wpa << 2); */
	printf("ttl=%u ", ntohs(icmp->icmp_lifetime));
	if (length < 8 + (icmp->icmp_num_addrs << 3)) {
	    printf("ERROR: Truncated ICMP: %u\n", length);
	    goto bad;
	}
	for (i = 0; i < icmp->icmp_num_addrs; i++) {
	    printf("addr%d=%s ", i + 1,
		   inet_ntoa(*(struct in_addr *)(packet + 8 + (i << 3))));
	    printf("(%d) ", (int) ntohl(*( (u32 *)(packet + 12 + (i << 3)) )));
	}
	packet += 8 + (icmp->icmp_num_addrs << 3);
	length -= 8 + (icmp->icmp_num_addrs << 3);
	break;
    case ICMP_TRACEROUTE:
	printf("id=%04x ", ntohs(icmp->icmp_trid));
	if (icmp->icmp_trvoid) {
	    printf("void=%04x ", ntohs(icmp->icmp_trvoid));
	}
	if (length < 20) {
	    printf("ERROR: Truncated ICMP traceroute: %u\n", length);
	    goto bad;
	}
	printf("ohc=%u rhc=%u ", ntohs(icmp->icmp_ohc), ntohs(icmp->icmp_rhc));
	printf("speed=%u ", ntohl(icmp->icmp_speed));
	printf("mtu=%u ", ntohl(icmp->icmp_mtu));
	packet += 20;
	length -= 20;
    default:
	packet += 4;
	length -= 4;
    }
    
    putchar('\n');

    if (length > 0) {
	dump_hex(packet, length);
    }
    
    return;

bad:
    /* Hex dump bad packets */
    dump_hex(packet, length);
    
    return;
}
