/*
 * (!c) 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *   Support newer IGMP types
 */

#include <stdio.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include "net/igmp.h"

#include "types.h"

void dump_hex(const u8 *packet, int length);
u16 in_cksum(u16 *addr, int length);

#ifndef IGMP_HDRLEN
#define IGMP_HDRLEN	8
#endif

void dump_igmp(const u8 *packet, int length)
{
    struct igmp_hdr *igmp = (struct igmp_hdr *) packet;
    u16 cksum;
    extern int numeric;
    
    printf(" IGMP: ");
    
#ifdef DEBUG    
    if (! packet) {
	printf("DEBUG: We got no packet!\n");
	return;
    }
#endif	    
    
    /* Check the length received */
    if (length < IGMP_HDRLEN) {
	printf("ERROR: Truncated IGMP: %u\n", length);
	goto bad;
    }
    
    /* Checksum */
    cksum = in_cksum((u16 *) igmp, length);
    if (cksum != 0) {
	printf("ERROR: Bad IGMP checksum\n");
	goto bad;
    }

    /* Print IGMP type */
    printf("type=");
    if (numeric) {
	printf("%u ", igmp->igmp_type);
    }
    else {
	switch (igmp->igmp_type) {
	case IGMP_MEMBERSHIP_QUERY:
	    printf("membership-query ");
	    break;
	case IGMP_V1_MEMBERSHIP_REPORT:
	    printf("membership-report-v1 ");
	    break;
	case IGMP_V2_MEMBERSHIP_REPORT:
	    printf("membership-report-v2 ");
	    break;
	case IGMP_V2_LEAVE_GROUP:
	    printf("leave-group ");
	    break;
	default:
	    printf("%u ", igmp->igmp_type);	    
	}
    }
    
    /* Print code */
    printf("code=%u ", igmp->igmp_code);

    /* Print group address */
    printf("group=%s\n", inet_ntoa(*(struct in_addr *)&igmp->igmp_group));
    
    packet += IGMP_HDRLEN;
    length -= IGMP_HDRLEN;
    
    /* Hex dump remaining data */
    dump_hex(packet, length);
    
    return;

bad:
    /* Hex dump bad packets */
    dump_hex(packet, length);
    
    return;
}
