/*
 * (!c) 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *   Better fragment handling
 *   XXX's
 */

#include <stdio.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "net/ip_protos.h"
#include "net/ip.h"

#include "types.h"

void dump_hex   (const u8 *packet, int length);
void dump_ascii (const u8 *packet, int length);
void dump_icmp  (const u8 *packet, int length);
void dump_igmp  (const u8 *packet, int length);
void dump_tcp   (const u8 *packet, int length, u32 ip_src, u32 ip_dst);
void dump_udp   (const u8 *packet, int length, u32 ip_src, u32 ip_dst);
void dump_ipopts(const u8 *packet, int length);
u16 in_cksum(u16 *addr, int length);
char *ip2name(u32 ip);

#ifndef IP_HDRLEN
#define IP_HDRLEN	20
#endif

void dump_ip(const u8 *packet, int length)
{
    struct ip_hdr *ip = (struct ip_hdr *) packet;
    int ip_hdrlen;	/* IP header length (in bytes) */
    int ip_len;		/* IP total length */
    int missing = 0;	/* missing bytes */    
    u16 ip_off;		/* IP frag offset + flags */
    u16 cksum;		/* IP Checksum */
    struct protoent *pe;
    extern int numeric;
    extern int resolve;

    printf("   IP: ");

#ifdef DEBUG    
    if (! packet) {
	printf("DEBUG: We got no packet!\n");
	return;
    }
#endif    

    /* Check the length received */
    if (length < IP_HDRLEN) {
	printf("ERROR: Truncated IP: %u\n", length);
	goto bad;
    }

    /* Check the IP version */
    if (ip->ip_v != IPVERSION) {
	/* XXX: What about IPv6 ? */
	printf("ERROR: Bad IP version: %u\n", ip->ip_v);
	goto bad;
    }    

    /* Check the IP header length */    
    ip_hdrlen = ip->ip_hl << 2;		/* ip_hl is in 32-bit words */
    
    if (ip_hdrlen < IP_HDRLEN) {
	printf("ERROR: ");
	printf("The IP header length is less than the minimal header: %u\n",
	       ip_hdrlen);
	goto bad;
    }

    /* Check the IP packet length */    
    ip_len = ntohs(ip->ip_len);		/* ip_len is in NBO */
    
    if (length < ip_len) {
	missing = ip_len - length;	/* XXX: what do we do with this ? */
    }
    else {
	/* length >= ip_len */
	length = ip_len;		/* avoid padding (e.g.: Ethernet) */
    }
    
    if (length < ip_hdrlen) {
	printf("ERROR: ");
	printf("The IP length is less than the IP header: %u\n", length);
	goto bad;
    }

    /* Checksum */
    cksum = in_cksum((u16 *) ip, ip_hdrlen);
    if (cksum != 0) {
	printf("ERROR: Bad IP checksum\n");
	goto bad;
    }
    
    if (resolve) {
	printf("%s ", ip2name(ip->ip_src));
	printf("-> %s ", ip2name(ip->ip_dst));
    }
    else {
	printf("%s ", inet_ntoa(*(struct in_addr *)&ip->ip_src));
	printf("-> %s ", inet_ntoa(*(struct in_addr *)&ip->ip_dst));	
    }

/*  printf("hlen=%u len=%u ", ip_hdrlen, ip_len); */
    printf("id=%u ", ntohs(ip->ip_id));    
    ip_off = ntohs(ip->ip_off);
    if (ip_off & IP_OFFMASK) {
	printf("off=%u ", ip_off & IP_OFFMASK);
    }
    if (ip_off & IP_RF) printf("RF ");
    if (ip_off & IP_DF) printf("DF ");
    if (ip_off & IP_MF) printf("MF ");
    printf("ttl=%u ", ip->ip_ttl);    
    if (ip->ip_tos) {
	printf("tos=%02x ", ip->ip_tos);
    }    

    switch (ip->ip_p) {
    case IPPROTO_ICMP:
    case IPPROTO_IGMP:
    case IPPROTO_TCP:
    case IPPROTO_UDP:
	putchar('\n');
	break;
    default:
	if (numeric) {
	    printf("proto=%02x\n", ip->ip_p);
	}
	else {
	    pe = getprotobynumber(ip->ip_p);
	    if (pe && pe->p_name) {
		printf("%s\n", pe->p_name);
	    }
	    else {
		printf("proto=%02x\n", ip->ip_p);
	    }
	}
    }

    /* 
     * Dump options
     */
    if (ip_hdrlen - IP_HDRLEN > 0) {
	dump_ipopts(packet + IP_HDRLEN, ip_hdrlen - IP_HDRLEN);
    }
    
    packet += ip_hdrlen;
    length -= ip_hdrlen;

    /*
     * Catch all fragmented packets (except the first)
     */
    if (ip_off & IP_OFFMASK) {
	/* XXX */
	if (length > 0) {
	    printf(" DATA: ");
	    dump_ascii(packet, length);
	}
	return;
    }
    
    /*
     * Dump higher level proto info
     */
    switch (ip->ip_p) {
    case IPPROTO_ICMP:
	dump_icmp(packet, length);
	break;
    case IPPROTO_IGMP:
	dump_igmp(packet, length);
	break;
    case IPPROTO_TCP:
	dump_tcp(packet, length, ip->ip_src, ip->ip_dst);
	break;
    case IPPROTO_UDP:
	dump_udp(packet, length, ip->ip_src, ip->ip_dst);
	break;
    default:
	dump_hex(packet, length);
    }

    return;
    
bad:
    /* Hex dump bad packets */
    dump_hex(packet, length);
    
    return;
}
