/*
 * (!c) 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *   More options
 */

#include <stdio.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include "net/ip.h"

#include "types.h"

void dump_hex(const u8 *packet, int length);
char *ts2asc(u32 ts);

void dump_ipopts(const u8 *packet, int length)
{
    int olen;	/* Option length */
    int tsflag;	/* Timestamp flag */
    int i, j;
    
    printf(" Opts: ");
    
    for (i = 0; i < length; ) {
	switch (*(packet + i)) {
	case IPOPT_EOL:		/* End of option list */
	    printf("eol ");
	    i++;
	    goto end;
	    /* NOTREACH */
	    break;
	case IPOPT_NOP:		/* No operation */
	    printf("nop ");
	    i++;
	    break;
#if 0	/* old (rfc791) */	    
	case IPOPT_SECURITY:	/* Security */
	    if ((i + 11 <= length) && (*(packet + i + 1) == 11)) {
		i += 2;
		printf("sec=");
		switch (ntohs(*( (u16 *)(packet + i) ))) {
		case IPOPT_SECUR_UNCLASS:
		    printf("unclass ");
		    break;
		case IPOPT_SECUR_CONFID:
		    printf("confid ");
		    break;
		case IPOPT_SECUR_EFTO:
		    printf("efto ");
		    break;
		case IPOPT_SECUR_MMMM:
		    printf("mmmm ");
		    break;
		case IPOPT_SECUR_RESTR:
		    printf("restr ");
		    break;
		case IPOPT_SECUR_SECRET:
		    printf("secret ");
		    break;
		case IPOPT_SECUR_TOPSECRET:
		    printf("topsec ");
		    break;
		default:
		    printf("%04x ", ntohs(*( (u16 *)(packet + i) )));
		}
		i += 2;
		printf("seccomp=%04x ", ntohs(*( (u16 *)(packet + i) )));
		i += 2;
		printf("sechand=%04x ", ntohs(*( (u16 *)(packet + i) )));
		i += 2;
		printf("sectcc=%04x ", ntohs(*( (u16 *)(packet + i) )));
		i += 2;
	    }
	    else {
		printf("ERROR: Invalid sec option.\n");
		goto bad;
	    }
	    break;
#else	/* new (rfc1108) */
	case IPOPT_SECURITY:	/* Security */
	    if ((i + 3 <= length) && ( (olen = *(packet + i + 1)) >= 3) &&
		(i + olen <= length)) {
		i += 2;
		printf("sec=");
		switch (*(packet + i)) {
		case IPOPT_SECUR_UNCLASS:
		    printf("unclass ");
		    break;
		case IPOPT_SECUR_CONFID:
		    printf("confid ");
		    break;
		case IPOPT_SECUR_SECRET:
		    printf("secret ");
		    break;
		case IPOPT_SECUR_TOPSECRET:
		    printf("topsec ");
		    break;
		default:
		    printf("%02x ", *(packet + i));
		}
		i++;
		printf("secauth=");
		for (j = 0; j < olen - 3; j++) {
		    printf("%02x", *(packet + i + j));
		}
		putchar(' ');
		i += olen - 3;
	    }
	    else {
		printf("ERROR: Invalid sec option.\n");
		goto bad;
	    }
	    break;
#endif
	case IPOPT_EXTENDEDSECURITY:	/* Security */
	    if ((i + 3 <= length) && ( (olen = *(packet + i + 1)) >= 3) &&
		(i + olen <= length)) {
		i += 2;
		printf("extsec=%02x ", *(packet + i));
		i++;
		printf("extsecinfo=");
		for (j = 0; j < olen - 3; j++) {
		    printf("%02x", *(packet + i + j));
		}
		putchar(' ');
		i += olen - 3;
	    }
	    else {
		printf("ERROR: Invalid extsec option.\n");
		goto bad;
	    }
	    break;
	case IPOPT_LSRR:	/* Louse Source and Record Route */
	    if ((i + 4 <= length) && ( (olen = *(packet + i + 1)) >= 4) &&
		(! (i % 4)) && (i + olen <= length)) {
		i += 4;
		printf("lsrr=");
		for (j = 0; j < olen - 4; j += 4) {
		    printf("%s ",
			   inet_ntoa(*(struct in_addr *)(packet + i + j)));
		}
		i += olen - 4;
	    }
	    else {
		printf("ERROR: Invalid lsrr option.\n");
		goto bad;
	    }
	    break;
	case IPOPT_SSRR:	/* Strict Source and Record Route */
	    if ((i + 4 <= length) && ( (olen = *(packet + i + 1)) >= 4) &&
		(! (i % 4)) && (i + olen <= length)) {
		i += 4;
		printf("ssrr=");
		for (j = 0; j < olen - 4; j += 4) {
		    printf("%s ",
			   inet_ntoa(*(struct in_addr *)(packet + i + j)));
		}
		i += olen - 4;
	    }
	    else {
		printf("ERROR: Invalid ssrr option.\n");
		goto bad;
	    }
	    break;
	case IPOPT_RR:		/* Record Route */
	    if ((i + 4 <= length) && ( (olen = *(packet + i + 1)) >= 4) &&
		(! (i % 4)) && (i + olen <= length)) {
		i += 4;
		printf("rr=");
		for (j = 0; j < olen - 4; j += 4) {
		    printf("%s ",
			   inet_ntoa(*(struct in_addr *)(packet + i + j)));
		}
		i += olen - 4;
	    }
	    else {
		printf("ERROR: Invalid rr option.\n");
		goto bad;
	    }
	    break;
	case IPOPT_SID:		/* Stream Identifier */
	    if ((i + 4 <= length) && (*(packet + i + 1) == 4)) {
		i += 2;
		printf("sid=%u ", ntohs(*( (u16 *)(packet + i) )));
		i += 2;
	    }
	    else {
		printf("ERROR: Invalid sid option.\n");
		goto bad;
	    }
	    break;
	case IPOPT_TRACEROUTE:	/* Traceroute */
	    if ((i + 12 <= length) && (*(packet + i + 1) == 12)) {
		i += 2;
		printf("trid=%u ", ntohs(*( (u16 *)(packet + i) )));
		i += 2;
		printf("trohc=%u ", ntohs(*( (u16 *)(packet + i) )));
		i += 2;
		printf("trrhc=%u ", ntohs(*( (u16 *)(packet + i) )));
		i += 2;
		printf("trip=%s ", inet_ntoa(*(struct in_addr *)(packet + i)));
		i += 4;
	    }
	    else {
		printf("ERROR: Invalid tr option.\n");
		goto bad;
	    }
	    break;
	case IPOPT_ROUTERALERT:	/* Router Alert */
	    if ((i + 4 <= length) && (*(packet + i + 1) == 4)) {
		i += 2;
		printf("ra=%u ", ntohs(*( (u16 *)(packet + i) )));
		i += 2;
	    }
	    else {
		printf("ERROR: Invalid ra option.\n");
		goto bad;
	    }
	    break;
	case IPOPT_TIMESTAMP:	/* Timestamp */
	    if ((i + 4 <= length) && ( (olen = *(packet + i + 1)) >= 4) &&
		(! (i % ( (tsflag = *(packet + i + 3) & 0x0f) ? 8 : 4))) &&
		(i + olen <= length)) {
		printf("tsof=%u ", *(packet + i + 3) & 0xf0);
		if (tsflag) {
		    i += 4;
		    for (j = 0; j < olen - 4; j += 8) {
			printf("tsip%d=%s ", j / 8 + 1,
			       inet_ntoa(*(struct in_addr *)(packet + i + j)));
			if (ntohl(*(u32 *)(packet + i + j + 4)) & 0x80000000) {
			    printf("ts%d=%08x ", j / 8 + 1,
				   ntohl(*(u32 *)(packet + i + j + 4)));
			}
			else {
			    printf("ts%d=%s ", j / 8 + 1,
				   ts2asc(*(u32 *)(packet + i + j + 4)));
			}
		    }
		}
		else {
		    i += 4;
		    for (j = 0; j < olen - 4; j += 4) {
			if (ntohl(*(u32 *)(packet + i + j)) & 0x80000000) {
			    printf("ts%d=%08x ", j / 4 + 1,
				   ntohl(*(u32 *)(packet + i + j)));
			}
			else {
			    printf("ts%d=%s ", j / 4 + 1,
				   ts2asc(*(u32 *)(packet + i + j + 4)));
			}
		    }
		}
		i += olen - 4;
	    }
	    else {
		printf("ERROR: Invalid ts option.\n");
		goto bad;
	    }
	    break;
	default:
	    goto bad;
	}
    }

end:
    putchar('\n');
    
    if (i < length) {
	dump_hex(packet + i, length - i);
    }
    
    return;
    
bad:
    dump_hex(packet + i, length - i);

    return;
}
