/*
 * (!c) 2002 by Ighighi
 * Venezuela
 */


#include <stdio.h>
#include <string.h>

#include <sys/socket.h>	/* AF_xxx */

#include "types.h"

void dump_hex(const u8 *packet, int length);
void dump_ip (const u8 *packet, int length);

/*
 * From tcpdump's print-null.c:
 * Byte-swap a 32-bit number.
 * htonl() or ntohl() won't work - we want to byte-swap even on
 * big-endian platforms
 */
#define SWAPLONG(x) \
((((x)&0xff)<<24) | (((x)&0xff00)<<8) | (((x)&0xff0000)>>8) | (((x)>>24)&0xff))

/*
 * From tcpdump's print-null.c:
 * The DLT_NULL packet header is 4 bytes long. It contains a host-byte order
 * 32-bit integer that specifies the family, e.g. AF_INET.
 * The OpenBSD DLT_LOOP packet header is the same, except that the integer
 * is in network byte order.
 */

#undef  NULL_HDRLEN
#define NULL_HDRLEN	4

void dump_null(const u8 *packet, int length)
{
    u32 family;

    printf(" NULL: ");

#ifdef DEBUG    
    if (! packet) {
	printf("DEBUG: We got no packet!\n");
	return;
    }
#endif	    

    /* Check the length received */
    if (length < NULL_HDRLEN) {
	printf("ERROR: Truncated NULL: %u\n", length);
	dump_hex(packet, length);
	return;
    }

    memcpy(&family, packet, sizeof(family));
    /*
     * Hack from tcpdump's print-null.c:
     * If the upper 16 bits aren't all zero, assume it's byte-swapped.
     */
    if (family & 0xffff0000) {
	family = SWAPLONG(family);
    }

    packet += NULL_HDRLEN;
    length -= NULL_HDRLEN;    

    switch (family) {
    case AF_INET:
	printf("inet\n");
	dump_ip(packet, length);
	break;
#if 0
#ifdef AF_INET6
    case AF_INET6:
	printf("inet6\n");
	dump_ip6(packet, length);
	break;
#endif
#endif
    default:
	printf("family=%04x\n", family);
	dump_hex(packet, length);	
    }
    
    return;
}
