/*
 * (!c) 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *   Dump higher level protocol info
 *   XXX's
 */

#include <stdio.h>

#include <netinet/in.h>		/* ntohs(), ntohl() */
#include <netdb.h>		/* getservbyport() */

#include "net/ip_protos.h"	/* IPPROTO_TCP */
#include "net/tcp.h"

#include "types.h"

void dump_hex    (const u8 *packet, int length);
void dump_ascii  (const u8 *packet, int length);
void dump_tcpopts(const u8 *packet, int length);
u16 tr_cksum(u16 *addr, int length, u32 ip_src, u32 ip_dst, u8 ip_p);

#undef	min
#define min(a, b)	(((a) > (b)) ? (b) : (a))

#ifndef TCP_HDRLEN
#define TCP_HDRLEN	20
#endif

void dump_tcp(const u8 *packet, int length, u32 ip_src, u32 ip_dst)
{
    struct tcp_hdr *tcp = (struct tcp_hdr *) packet;
    int tcp_hdrlen;	/* TCP header length (in bytes) */
    int tcp_urp;	/* TCP urgent pointer */
    int urgent;		/* urgent bytes */
    u16 cksum;		/* TCP Checksum */
    struct servent *se;
    extern int numeric;
    
    printf("  TCP: ");    

#ifdef DEBUG
    if (! packet) {
	printf("DEBUG: We got no packet!\n");
	return;
    }
#endif

    /* Check the length received */
    if (length < TCP_HDRLEN) {
	printf("ERROR: Truncated IP: %u\n", length);
	goto bad;
    }

    /* Check the TCP header length */    
    tcp_hdrlen = tcp->th_off << 2;	/* th_off is in 32-bit words */

    if (tcp_hdrlen < TCP_HDRLEN) {
	printf("ERROR: ");
	printf("THe TCP header length is less than the minimal header: %u\n",
	       tcp_hdrlen);
	goto bad;
    }

    /* Check the length */
    if (length < tcp_hdrlen) {
	printf("ERROR: ");
	printf("The length received is less than the TCP header: %u\n",
	       length);
	goto bad;
    }
    
    /* Checksum */
    cksum = tr_cksum((u16 *) tcp, length, ip_src, ip_dst, IPPROTO_TCP);
    if (cksum != 0) {
	printf("ERROR: Bad TCP checksum\n");
	goto bad;
    }
    
    if (numeric) {
	printf("%u -> %u ", ntohs(tcp->th_sport), ntohs(tcp->th_dport));
    }
    else {
	se = getservbyport(tcp->th_sport, "tcp");
	if (se && se->s_name) {
	    printf("%s ", se->s_name);
	}
	else {
	    printf("%u ", ntohs(tcp->th_sport));
	}
	se = getservbyport(tcp->th_dport, "tcp");
	if (se && se->s_name) {
	    printf("-> %s ", se->s_name);
	}
	else {
	    printf("-> %u ", ntohs(tcp->th_dport));
	}
    }
/*  printf("hlen=%u ", tcp_hdrlen); */
    if (tcp->th_flags & TH_CWR)  putchar('C'); else putchar('-');
    if (tcp->th_flags & TH_ECE)  putchar('E'); else putchar('-');
    if (tcp->th_flags & TH_URG)  putchar('U'); else putchar('-');
    if (tcp->th_flags & TH_ACK)  putchar('A'); else putchar('-');
    if (tcp->th_flags & TH_PUSH) putchar('P'); else putchar('-');
    if (tcp->th_flags & TH_RST)  putchar('R'); else putchar('-');
    if (tcp->th_flags & TH_SYN)  putchar('S'); else putchar('-');
    if (tcp->th_flags & TH_FIN)  putchar('F'); else putchar('-');
    printf(" %u/%u", ntohl(tcp->th_seq), ntohl(tcp->th_ack));
    printf(" win=%u", ntohs(tcp->th_win));
    tcp_urp = ntohs(tcp->th_urp);    
    if (tcp_urp || (tcp->th_flags & TH_URG)) {
	printf(" urg=%u", tcp_urp);
    }
    if (tcp->th_x2) {
	printf(" x2=%x", tcp->th_x2);
    }
    putchar('\n');

    /*
     * Dump options
     */
    if (tcp_hdrlen - TCP_HDRLEN > 0) {
	dump_tcpopts(packet + TCP_HDRLEN, tcp_hdrlen - TCP_HDRLEN);
    }

    packet += tcp_hdrlen;
    length -= tcp_hdrlen;

    /*
     * Dump urgent data
     */
    if (tcp->th_flags & TH_URG) {
	urgent = min(tcp_urp, length);	/* Avoid IP fragmentation */
	if (urgent) {
	    printf("  URG: ");
	    dump_ascii(packet, urgent);
	    packet += urgent;
	    length -= urgent;
	}
    }

    /*
     * XXX: Dump higher level protocol info
     */
    
    if (length > 0) {
	printf(" DATA: ");
	dump_ascii(packet, length);
    }
    
    return;
    
bad:
    /* Hex dump bad packets */
    dump_hex(packet, length);
    
    return;
}
