/*
 * (!c) 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *   More options
 *   XXX's
 */

#include <stdio.h>

#include <netinet/in.h>		/* ntohs() */

#include "net/tcp.h"

#include "types.h"

void dump_hex(const u8 *packet, int length);

void dump_tcpopts(const u8 *packet, int length)
{
    int olen;	/* Option length */
    int i, j;
    
    printf(" Opts: ");
    
    for (i = 0; i < length; ) {
	switch (*(packet + i)) {
	case TCPOPT_EOL:	/* End of option list */
	    printf("eol ");
	    i++;
	    goto end;
	    /* NOTREACH */
	    break;
	case TCPOPT_NOP:	/* No operation */
	    printf("nop ");
	    i++;
	    break;
	case TCPOPT_MAXSEG:	/* Maximum segment size */
	    if ((i + TCPOLEN_MAXSEG <= length) &&
		(*(packet + i + 1) == TCPOLEN_MAXSEG)) {
		i += 2;
		printf("mss=%u ", ntohs(*( (u16 *)(packet + i) )));
		i += 2;
	    }
	    else {
		printf("ERROR: Invalid mss option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_WSCALE:	/* Window scale */
	    if ((i + TCPOLEN_WSCALE <= length) &&
		(*(packet + i + 1) == TCPOLEN_WSCALE)) {
		i += 2;
		printf("ws=%u ", *(packet + i));
		i++;
	    }
	    else {
		printf("ERROR: Invalid ws option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_SACKOK:	/* SACK permitted */
	    if ((i + TCPOLEN_SACKOK <= length) &&
		(*(packet + i + 1) == TCPOLEN_SACKOK)) {
		printf("sackok ");
		i += 2;
	    }
	    else {
		printf("ERROR: Invalid sackok option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_SACK:	/* Selective Acknowledgment */
	    if ((i + 2 <= length) &&
		( (olen = *(packet + i + 1)) >= 2) &&	/* XXX */
		(i + olen <= length)) {
		i += 2;
		printf("sack=");
		for (j = 0; j < olen - 2; j++) {
		    printf("%02x", *(packet + i + j));
		}
		putchar(' ');
		i += olen - 2;
	    }
	    else {
		printf("ERROR: Invalid sack option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_ECHO:	/* Echo */
	    if ((i + TCPOLEN_ECHO <= length) &&
		(*(packet + i + 1) == TCPOLEN_ECHO)) {
		i += 2;
		printf("echo=%u ", ntohl(*( (u32 *)(packet + i) )));
		i += 4;
	    }
	    else {
		printf("ERROR: Invalid echo option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_ECHOREPLY:	/* Echo Reply */
	    if ((i + TCPOLEN_ECHOREPLY <= length) &&
		(*(packet + i + 1) == TCPOLEN_ECHOREPLY)) {
		i += 2;
		printf("echoreply=%u ", ntohl(*( (u32 *)(packet + i) )));
		i += 4;
	    }
	    else {
		printf("ERROR: Invalid echoreply option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_TIMESTAMP:	/* Timestamp */
	    if ((i + TCPOLEN_TIMESTAMP <= length) &&
		(*(packet + i + 1) == TCPOLEN_TIMESTAMP)) {
		i += 2;
		printf("tsval=%u ", ntohl(*( (u32 *)(packet + i) )));
		i += 4;
		printf("tsecr=%u ", ntohl(*( (u32 *)(packet + i) )));
		i += 4;
	    }
	    else {
		printf("ERROR: Invalid ts option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_CC:		/* Connection Count */
	    if ((i + TCPOLEN_CC <= length) &&
		(*(packet + i + 1) == TCPOLEN_CC)) {
		i += 2;
		printf("cc=%u ", ntohl(*( (u32 *)(packet + i) )));
		i += 4;
	    }
	    else {
		printf("ERROR: Invalid cc option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_CCNEW:	/* CC New */
	    if ((i + TCPOLEN_CCNEW <= length) &&
		(*(packet + i + 1) == TCPOLEN_CCNEW)) {
		i += 2;
		printf("ccnew=%u ", ntohl(*( (u32 *)(packet + i) )));
		i += 4;
	    }
	    else {
		printf("ERROR: Invalid ccnew option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_CCECHO:	/* CC Echo */
	    if ((i + TCPOLEN_CCECHO <= length) &&
		(*(packet + i + 1) == TCPOLEN_CCECHO)) {
		i += 2;
		printf("ccecho=%u ", ntohl(*( (u32 *)(packet + i) )));
		i += 4;
	    }
	    else {
		printf("ERROR: Invalid ccecho option.\n");
		goto bad;
	    }
	    break;
	case TCPOPT_MD5:	/* MD5 signature */
	    if ((i + TCPOLEN_MD5 <= length) &&
		(*(packet + i + 1) == TCPOLEN_MD5)) {
		i += 2;
		for (j = 0; j < TCPOLEN_MD5 - 2; j++) {
		    printf("%02x", *(packet + i + j));
		}
		i += TCPOLEN_MD5 - 2;
	    }
	    else {
		printf("ERROR: Invalid md5 option.\n");
		goto bad;
	    }
	    break;
	default:
	    goto bad;
	}
    }

end:
    putchar('\n');
    
    if (i < length) {
	dump_hex(packet + i, length - i);
    }
    
    return;
    
bad:
    dump_hex(packet + i, length - i);
    
    return;
}
