/*
 * (!c) 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *   Dump higher level protocol info
 *   XXX's
 */

#include <stdio.h>

#include <netinet/in.h>		/* ntohs() */
#include <netdb.h>		/* getservbyport() */

#include "net/ip_protos.h"	/* IPPROTO_UDP */
#include "net/udp.h"

#include "types.h"

void dump_hex  (const u8 *packet, int length);
void dump_ascii(const char *packet, int length);
u16 tr_cksum(u16 *addr, int length, u32 ip_src, u32 ip_dst, u8 ip_p);

#ifndef UDP_HDRLEN
#define UDP_HDRLEN	8
#endif

void dump_udp(const u8 *packet, int length, u32 ip_src, u32 ip_dst)
{
    struct udp_hdr *udp = (struct udp_hdr *) packet;
    int udp_len;	/* UDP length */
    int missing = 0;	/* missing bytes */
    u16 cksum;		/* UDP Checksum */
    struct servent *se;
    extern int numeric;

    printf("  UDP: ");

#ifdef DEBUG
    if (! packet) {
	printf("DEBUG: We got no packet!\n");
	return;
    }
#endif

    /* Check the length received */
    if (length < UDP_HDRLEN) {
	printf("ERROR: Truncated UDP: %u\n", length);
	goto bad;
    }

    /* Check the UDP packet length */
    udp_len = ntohs(udp->uh_ulen);
    
    if (length < udp_len) {
	missing = udp_len - length;	/* XXX: what do we do with this ? */
    }
    else {
	/* length >= udp_len */	
	/* XXX: should we note this ? */
	length = udp_len;
    }
    
    if (length < UDP_HDRLEN) {
	printf("ERROR: ");
	printf("The UDP length is less than the UDP header: %u\n", length);
	goto bad;
    }

    /* Checksum (in UDP it is optional...) */
    cksum = (udp->uh_sum ?
	     tr_cksum((u16 *) udp, length, ip_src, ip_dst, IPPROTO_UDP) :
	     0);
    if (cksum != 0) {
	printf("ERROR: Bad UDP checksum\n");
	goto bad;
    }    
    
    if (numeric) {
	printf("%u -> %u ", ntohs(udp->uh_sport), ntohs(udp->uh_dport));
    }
    else {
	se = getservbyport(udp->uh_sport, "udp");
	if (se && se->s_name) {
	    printf("%s ", se->s_name);
	}
	else {
	    printf("%u ", ntohs(udp->uh_sport));
	}
	se = getservbyport(udp->uh_dport, "udp");
	if (se && se->s_name) {
	    printf("-> %s ", se->s_name);
	}
	else {
	    printf("-> %u ", ntohs(udp->uh_dport));
	}
    }
/*  printf("len=%u ", udp_len); */
    putchar('\n');

    packet += UDP_HDRLEN;
    length -= UDP_HDRLEN;
    
    /*
     * XXX: Dump higher level protocol info
     */
    
    if (length > 0) {
	printf(" DATA: ");
	dump_ascii(packet, length);
    }
    
    return;
    
bad:
    /* Hex dump bad packets */
    dump_hex(packet, length);
    
    return;
}
