/*
 * (!c) 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *   DLT_PPP_BSDOS
 *   DLT_SLIP_BSDOS
 *   DLT_LINUX_SLL
 *   DLT_IEEE802
 *   DLT_FDDI
 *   DLT_xxx
 */

#include <stdio.h>

#include "pcap.h"

#include "types.h"

/* 
 * handler prototypes (`user' not used)
 */
void ether_handler(u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ppp_handler  (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void slip_handler (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void raw_handler  (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void null_handler (u_char *user, const struct pcap_pkthdr *h, const u_char *p);

/*
 * dump prototypes
 */

void dump_time (const struct timeval *now);
void dump_ether(const u8 *packet, int length);
void dump_ppp  (const u8 *packet, int length);
void dump_slip (const u8 *packet, int length);
void dump_raw  (const u8 *packet, int length);
void dump_null (const u8 *packet, int length);

struct dlt_handler {
    int dlt;
    pcap_handler handler;
};
    
struct dlt_handler handlers[] = {    
#ifdef DLT_EN10MB    
    {  DLT_EN10MB,	ether_handler },
#endif
#ifdef DLT_PPP    
    {  DLT_PPP,		ppp_handler },
#endif
#ifdef DLT_SLIP
    {  DLT_SLIP,	slip_handler },
#endif    
#ifdef DLT_RAW    
    {  DLT_RAW,		raw_handler },
#endif
#ifdef DLT_NULL
    {  DLT_NULL,	null_handler },
#endif
#ifdef DLT_LOOP    
    {  DLT_LOOP,	null_handler },
#endif
    {  0,		NULL }
};

pcap_handler lookup_handler(int dlt)
{
    struct dlt_handler *hp;

    for (hp = handlers; hp->handler; ++hp) {
	if (dlt == hp->dlt) {
	    return (hp->handler);
	}
    }
    
    return ((pcap_handler) NULL);
}
	
/*
 * From <pcap.h>:
 *   struct pcap_pkthdr *h;
 *   struct timeval	h->ts:		time stamp
 *   bpf_uint32		h->caplen:	length of portion present
 *   bpf_uint32		h->len:		length this packet (off wire)
 */

#ifdef DLT_EN10MB
/*
 * Used for 10/100 Mbps Ethernet
 * Used for the loopback interface on Linux
 *   (DLT_NULL & DLT_LOOP is used by others; see below)
 */
void ether_handler(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
{
#ifdef __linux__
    extern int avoid_duplicate;
    static int duplicate = 0;
#endif    
    u_int caplen = h->caplen;

#ifdef __linux__
    if (avoid_duplicate) {
	if (duplicate) {
	    duplicate = 0;
	    return;
	}
	else {
	    duplicate = 1;
	}
    }
#endif    
    
#ifdef DEBUG
    if (caplen != h->len) {
	printf("DEBUG: caplen: %d, len: %d\n", h->caplen, h->len);
    }
#endif

    dump_time(&(h->ts));
    dump_ether(p, caplen);
    
    return;
}
#endif	/* DLT_EN10MB */

#ifdef DLT_PPP
/*
 * Used for PPP on some OSs (DLT_RAW is used by others; see below)
 */
void ppp_handler(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
{
    u_int caplen = h->caplen;
    
#ifdef DEBUG
    if (caplen != h->len) {
	printf("DEBUG: caplen: %d, len: %d\n", h->caplen, h->len);
    }
#endif

    dump_time(&(h->ts));
    dump_ppp(p, caplen);
    
    return;
}
#endif	/* DLT_PPP */

#ifdef DLT_SLIP
/*
 * Used for SLIP on some OSs (DLT_RAW is used by others; see below)
 */
void slip_handler(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
{
    u_int caplen = h->caplen;
    
#ifdef DEBUG
    if (caplen != h->len) {
	printf("DEBUG: caplen: %d, len: %d\n", h->caplen, h->len);
    }
#endif

    dump_time(&(h->ts));
    dump_slip(p, caplen);
    
    return;
}
#endif	/* DLT_SLIP */

#ifdef DLT_RAW
/*
 * Used for PPP / SLIP on some OSs (Linux & Irix)
 */
void raw_handler(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
{
    u_int caplen = h->caplen;
    
#ifdef DEBUG
    if (caplen != h->len) {
	printf("DEBUG: caplen: %d, len: %d\n", h->caplen, h->len);
    }
#endif

    dump_time(&(h->ts));
    dump_raw(p, caplen);
    
    return;
}
#endif	/* DLT_RAW */

#if defined(DLT_NULL) || defined(DLT_LOOP)
/*
 * Used for the loopback interface on some OSs (Linux uses DLT_EN10MB)
 *   (DLT_LOOP is used on OpenBSD)
 */
void null_handler(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
{
    u_int caplen = h->caplen;
    
#ifdef DEBUG
    if (caplen != h->len) {
	printf("DEBUG: caplen: %d, len: %d\n", h->caplen, h->len);
    }
#endif

    dump_time(&(h->ts));
    dump_null(p, caplen);
    
    return;
}
#endif	/* DLT_NULL || DLT_LOOP */
